import numpy as np
import matplotlib.pyplot as plt
from sklearn.linear_model import LinearRegression
from sklearn.pipeline import make_pipeline
from sklearn.preprocessing import PolynomialFeatures
from sklearn.metrics import mean_squared_error, r2_score
from sklearn.model_selection import train_test_split

# 3차 다항식 데이터 생성
np.random.seed(0)
n_samples = 30
X = np.sort(np.random.rand(n_samples))
y = np.sin(2 * np.pi * X) + 0.1 * np.random.randn(n_samples)

# 학습 데이터셋과 테스트 데이터셋 분리
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2)

# Polynomial 회귀 모델 학습 및 예측 (Underfitting)
degree = 1
model = make_pipeline(PolynomialFeatures(degree), LinearRegression())
model.fit(X_train.reshape(-1, 1), y_train)
y_pred_train = model.predict(X_train.reshape(-1, 1))
y_pred_test = model.predict(X_test.reshape(-1, 1))

# MSE 및 R2-score 계산
mse_train = mean_squared_error(y_train, y_pred_train)
mse_test = mean_squared_error(y_test, y_pred_test)
r2_train = r2_score(y_train, y_pred_train)
r2_test = r2_score(y_test, y_pred_test)

# 그래프 그리기
plt.figure(figsize=(10, 5))
plt.subplot(1, 2, 1)
plt.scatter(X_train, y_train, s=20, edgecolor="black", c="darkorange")
plt.plot(X_train, y_pred_train, color="cornflowerblue", label=f"degree={degree}\nMSE={mse_train:.2f}, R2={r2_train:.2f}")
plt.title("Underfitting (High Bias)", fontsize=14)
plt.xlabel("X")
plt.ylabel("y")
plt.legend()

# Polynomial 회귀 모델 학습 및 예측 (Overfitting)
degree = 10
model = make_pipeline(PolynomialFeatures(degree), LinearRegression())
model.fit(X_train.reshape(-1, 1), y_train)
y_pred_train = model.predict(X_train.reshape(-1, 1))
y_pred_test = model.predict(X_test.reshape(-1, 1))

# MSE 및 R2-score 계산
mse_train = mean_squared_error(y_train, y_pred_train)
mse_test = mean_squared_error(y_test, y_pred_test)
r2_train = r2_score(y_train, y_pred_train)
r2_test = r2_score(y_test, y_pred_test)

# 그래프 그리기
plt.subplot(1, 2, 2)
plt.scatter(X_train, y_train, s=20, edgecolor="black", c="darkorange")
plt.plot(X_train, y_pred_train, color="cornflowerblue", label=f"degree={degree}\nMSE={mse_train:.2f}, R2={r2_train:.2f}")
plt.title("Overfitting (High Variance)", fontsize=14)
plt.xlabel("X")
plt.ylabel("y")
plt.legend()

plt.tight_layout()
plt.show()